@@ -108,7 +108,7 @@ namespace rstan {
         sampling_algo_t algorithm;
         int warmup; // number of warmup
         int thin;
-        bool save_warmup; // weather to save warmup samples (always true now)
+        bool save_warmup; // weather to save warmup samples (true by default)
         int iter_save; // number of iterations saved
         int iter_save_wo_warmup; // number of iterations saved wo warmup
         bool adapt_engaged;
@@ -145,7 +145,9 @@ namespace rstan {
         int elbo_samples; // default to 100
         int eval_elbo;    // default to 100
         int output_samples; // default to 1000
-        double eta_adagrad; // default to 0.1
+        double eta; // defaults to 1.0
+        bool adapt_engaged; // defaults to 1
+        int adapt_iter; // defaults to 50
         double tol_rel_obj; // default to 0.01
       } variational;
       struct {
@@ -172,7 +174,7 @@ namespace rstan {
           }
           if (ctrl.sampling.adapt_delta <= 0 || ctrl.sampling.adapt_delta >= 1) {
             std::stringstream msg;
-            msg << "Invalid adaptation parameter (found detal="
+            msg << "Invalid adaptation parameter (found delta="
                 << ctrl.sampling.adapt_delta << "; require 0<delta<1).";
             throw std::invalid_argument(msg.str());
           }
@@ -235,12 +237,6 @@ namespace rstan {
                 << ctrl.variational.elbo_samples << "; require 0 < elbo_samples).";
             throw std::invalid_argument(msg.str());
           }
-          if (ctrl.variational.eta_adagrad < 0 || ctrl.variational.eta_adagrad > 1) {
-            std::stringstream msg;
-            msg << "Invalid parameter eta_adagrad (found eta_adagrad="
-                << ctrl.variational.eta_adagrad << "; require 0 < eta_adagrad <= 1).";
-            throw std::invalid_argument(msg.str());
-          }
           if (ctrl.variational.iter <= 0) {
             std::stringstream msg;
             msg << "Invalid parameter iter (found iter="
@@ -253,6 +249,12 @@ namespace rstan {
                 << ctrl.variational.tol_rel_obj << "; require 0 < tol_rel_obj).";
             throw std::invalid_argument(msg.str());
           }
+          if (ctrl.variational.eta <= 0) {
+            std::stringstream msg;
+            msg << "Invalid parameter eta (found eta="
+                << ctrl.variational.eta << "; require 0 < eta).";
+            throw std::invalid_argument(msg.str());
+          }
           if (ctrl.variational.eval_elbo <= 0) {
             std::stringstream msg;
             msg << "Invalid parameter eval_elbo (found eval_elbo="
@@ -265,6 +267,12 @@ namespace rstan {
                 << ctrl.variational.output_samples << "; require 0 < output_samples).";
             throw std::invalid_argument(msg.str());
           }
+          if (ctrl.variational.adapt_iter <= 0) {
+            std::stringstream msg;
+            msg << "Invalid parameter adapt_iter (found adapt_iter="
+                << ctrl.variational.adapt_iter << "; require 0 < adapt_iter).";
+            throw std::invalid_argument(msg.str());
+          }
           break;
       }
     }
@@ -304,7 +312,9 @@ namespace rstan {
           get_rlist_element(in, "elbo_samples", ctrl.variational.elbo_samples, 100);
           get_rlist_element(in, "eval_elbo", ctrl.variational.eval_elbo, 100);
           get_rlist_element(in, "output_samples", ctrl.variational.output_samples, 1000);
-          get_rlist_element(in, "eta_adagrad", ctrl.variational.eta_adagrad, 0.1);
+          get_rlist_element(in, "adapt_iter", ctrl.variational.adapt_iter, 50);
+          get_rlist_element(in, "eta", ctrl.variational.eta, 1.0);
+          get_rlist_element(in, "adapt_engaged", ctrl.variational.adapt_engaged, true);
           get_rlist_element(in, "tol_rel_obj", ctrl.variational.tol_rel_obj, 0.01);
           ctrl.variational.algorithm = MEANFIELD;
           if (get_rlist_element(in, "algorithm", t_str)) {
@@ -314,6 +324,7 @@ namespace rstan {
         case SAMPLING:
           get_rlist_element(in, "iter", ctrl.sampling.iter, 2000);
           get_rlist_element(in, "warmup", ctrl.sampling.warmup, ctrl.sampling.iter / 2);
+          get_rlist_element(in, "save_warmup", ctrl.sampling.save_warmup, true);
 
           calculated_thin = (ctrl.sampling.iter - ctrl.sampling.warmup) / 1000;
           if (calculated_thin < 1) calculated_thin = 1;
@@ -322,8 +333,10 @@ namespace rstan {
           ctrl.sampling.iter_save_wo_warmup
             = 1 + (ctrl.sampling.iter - ctrl.sampling.warmup - 1) / ctrl.sampling.thin;
           ctrl.sampling.iter_save
-            = ctrl.sampling.iter_save_wo_warmup
-              + 1 + (ctrl.sampling.warmup - 1) / ctrl.sampling.thin;
+            = ctrl.sampling.iter_save_wo_warmup;
+          if (ctrl.sampling.save_warmup)
+            ctrl.sampling.iter_save += 
+              1 + (ctrl.sampling.warmup - 1) / ctrl.sampling.thin;
 
           ctrl.sampling.refresh = (ctrl.sampling.iter >= 20) ?
                                   ctrl.sampling.iter / 10 : 1;
@@ -459,8 +472,10 @@ namespace rstan {
           args["elbo_samples"] = Rcpp::wrap(ctrl.variational.elbo_samples);
           args["eval_elbo"] = Rcpp::wrap(ctrl.variational.eval_elbo);
           args["output_samples"] = Rcpp::wrap(ctrl.variational.output_samples);
-          args["eta_adagrad"] = Rcpp::wrap(ctrl.variational.eta_adagrad);
+          args["eta"] = Rcpp::wrap(ctrl.variational.eta);
+          args["adapt_engaged"] = Rcpp::wrap(ctrl.variational.adapt_engaged);
           args["tol_rel_obj"] = Rcpp::wrap(ctrl.variational.tol_rel_obj);
+          args["adapt_iter"] = Rcpp::wrap(ctrl.variational.adapt_iter);
           switch (ctrl.variational.algorithm) {
             case MEANFIELD: args["algorithm"] = Rcpp::wrap("meanfield"); break;
             case FULLRANK: args["algorithm"] = Rcpp::wrap("fullrank"); break;
@@ -473,6 +488,7 @@ namespace rstan {
           args["thin"] = Rcpp::wrap(ctrl.sampling.thin);
           args["refresh"] = Rcpp::wrap(ctrl.sampling.refresh);
           args["test_grad"] = Rcpp::wrap(false);
+          args["save_warmup"] = Rcpp::wrap(ctrl.sampling.save_warmup);
           ctrl_args["adapt_engaged"] = Rcpp::wrap(ctrl.sampling.adapt_engaged);
           ctrl_args["adapt_gamma"] = Rcpp::wrap(ctrl.sampling.adapt_gamma);
           ctrl_args["adapt_delta"] = Rcpp::wrap(ctrl.sampling.adapt_delta);
@@ -585,8 +601,11 @@ namespace rstan {
     inline int get_ctrl_variational_eval_elbo() const {
       return ctrl.variational.eval_elbo;
     }
-    inline double get_ctrl_variational_eta_adagrad() const {
-      return ctrl.variational.eta_adagrad;
+    inline double get_ctrl_variational_eta() const {
+      return ctrl.variational.eta;
+    }
+    inline bool get_ctrl_variational_adapt_engaged() const {
+      return ctrl.variational.adapt_engaged;
     }
     inline double get_ctrl_variational_tol_rel_obj() const {
       return ctrl.variational.tol_rel_obj;
@@ -594,7 +613,10 @@ namespace rstan {
     inline variational_algo_t get_ctrl_variational_algorithm() const {
       return ctrl.variational.algorithm;
     }
-
+    inline int get_ctrl_variational_adapt_iter() const {
+      return ctrl.variational.adapt_iter;
+    }
+    
     inline int get_ctrl_sampling_refresh() const {
       return ctrl.sampling.refresh;
     }
@@ -671,7 +693,7 @@ namespace rstan {
        return ctrl.sampling.iter_save;
     }
     inline bool get_ctrl_sampling_save_warmup() const {
-       return true;
+       return ctrl.sampling.save_warmup; // was true
     }
     inline optim_algo_t get_ctrl_optim_algorithm() const {
       return ctrl.optim.algorithm;
@@ -737,7 +759,7 @@ namespace rstan {
           write_comment_property(ostream,"elbo_samples", ctrl.variational.elbo_samples);
           write_comment_property(ostream,"output_samples", ctrl.variational.output_samples);
           write_comment_property(ostream,"eval_elbo", ctrl.variational.eval_elbo);
-          write_comment_property(ostream,"eta_adagrad", ctrl.variational.eta_adagrad);
+          write_comment_property(ostream,"eta", ctrl.variational.eta);
           write_comment_property(ostream,"tol_rel_obj", ctrl.variational.tol_rel_obj);
           switch (ctrl.variational.algorithm) {
             case MEANFIELD: write_comment_property(ostream,"algorithm", "meanfield"); break;
@@ -746,7 +768,7 @@ namespace rstan {
           break;
         case SAMPLING:
           write_comment_property(ostream,"warmup",ctrl.sampling.warmup);
-          write_comment_property(ostream,"save_warmup",1);
+          write_comment_property(ostream,"save_warmup",ctrl.sampling.save_warmup);
           write_comment_property(ostream,"thin",ctrl.sampling.thin);
           write_comment_property(ostream,"refresh",ctrl.sampling.refresh);
           write_comment_property(ostream,"stepsize",ctrl.sampling.stepsize);
